@@ -44,9 +44,9 @@ OPTS = {'base_rules': [policy.RuleDefault('admin', 'is_admin:True',
         }
 
 
-class GenerateSampleTestCase(base.PolicyBaseTestCase):
+class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
     def setUp(self):
-        super(GenerateSampleTestCase, self).setUp()
+        super(GenerateSampleYAMLTestCase, self).setUp()
         self.enforcer = policy.Enforcer(self.conf, policy_file='policy.yaml')
 
     def _capture_stdout(self):
@@ -195,6 +195,109 @@ class GenerateSampleTestCase(base.PolicyBaseTestCase):
         self.assertEqual(expected, written_policy)
 
 
+class GenerateSampleJSONTestCase(base.PolicyBaseTestCase):
+    def setUp(self):
+        super(GenerateSampleJSONTestCase, self).setUp()
+        self.enforcer = policy.Enforcer(self.conf, policy_file='policy.json')
+
+    def _capture_stdout(self):
+        self.useFixture(fixtures.MonkeyPatch('sys.stdout', moves.StringIO()))
+        return sys.stdout
+
+    def test_generate_loadable_json(self):
+        extensions = []
+        for name, opts in OPTS.items():
+            ext = stevedore.extension.Extension(name=name, entry_point=None,
+                                                plugin=None, obj=opts)
+            extensions.append(ext)
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=extensions, namespace=['base_rules', 'rules'])
+
+        output_file = self.get_config_file_fullname('policy.json')
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr) as mock_ext_mgr:
+            # generate sample-policy file with only rules
+            generator._generate_sample(['base_rules', 'rules'], output_file,
+                                       output_format='json',
+                                       include_help=False)
+            mock_ext_mgr.assert_called_once_with(
+                'oslo.policy.policies', names=['base_rules', 'rules'],
+                on_load_failure_callback=generator.on_load_failure_callback,
+                invoke_on_load=True)
+
+        self.enforcer.load_rules()
+
+        self.assertIn('owner', self.enforcer.rules)
+        self.assertIn('admin', self.enforcer.rules)
+        self.assertIn('admin_or_owner', self.enforcer.rules)
+        self.assertEqual('project_id:%(project_id)s',
+                         str(self.enforcer.rules['owner']))
+        self.assertEqual('is_admin:True', str(self.enforcer.rules['admin']))
+        self.assertEqual('(rule:admin or rule:owner)',
+                         str(self.enforcer.rules['admin_or_owner']))
+
+    def test_expected_content(self):
+        extensions = []
+        for name, opts in OPTS.items():
+            ext = stevedore.extension.Extension(name=name, entry_point=None,
+                                                plugin=None, obj=opts)
+            extensions.append(ext)
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=extensions, namespace=['base_rules', 'rules'])
+
+        expected = '''{
+    "admin": "is_admin:True",
+    "owner": "project_id:%(project_id)s",
+    "shared": "field:networks:shared=True",
+    "admin_or_owner": "rule:admin or rule:owner"
+}
+'''
+        output_file = self.get_config_file_fullname('policy.json')
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr) as mock_ext_mgr:
+            generator._generate_sample(['base_rules', 'rules'],
+                                       output_file=output_file,
+                                       output_format='json')
+            mock_ext_mgr.assert_called_once_with(
+                'oslo.policy.policies', names=['base_rules', 'rules'],
+                on_load_failure_callback=generator.on_load_failure_callback,
+                invoke_on_load=True)
+
+        with open(output_file, 'r') as written_file:
+            written_policy = written_file.read()
+
+        self.assertEqual(expected, written_policy)
+
+    def test_expected_content_stdout(self):
+        extensions = []
+        for name, opts in OPTS.items():
+            ext = stevedore.extension.Extension(name=name, entry_point=None,
+                                                plugin=None, obj=opts)
+            extensions.append(ext)
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=extensions, namespace=['base_rules', 'rules'])
+
+        expected = '''{
+    "admin": "is_admin:True",
+    "owner": "project_id:%(project_id)s",
+    "shared": "field:networks:shared=True",
+    "admin_or_owner": "rule:admin or rule:owner"
+}
+'''
+        stdout = self._capture_stdout()
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr) as mock_ext_mgr:
+            generator._generate_sample(['base_rules', 'rules'],
+                                       output_file=None,
+                                       output_format='json')
+            mock_ext_mgr.assert_called_once_with(
+                'oslo.policy.policies', names=['base_rules', 'rules'],
+                on_load_failure_callback=generator.on_load_failure_callback,
+                invoke_on_load=True)
+
+        self.assertEqual(expected, stdout.getvalue())
+
+
 class GeneratorRaiseErrorTestCase(testtools.TestCase):
     def test_generator_raises_error(self):
         """Verifies that errors from extension manager are not suppressed."""
